﻿using System;
using System.IO;
using System.Drawing;

/// <summary>
/// Класс, описывающий информацию о знакоместе (символе)
/// </summary>
[Serializable]
public class LcdChar
{/// <summary>
/// Указывает принадлежность знакоместа к пользовательским
/// </summary>
    public bool User { get; set; }
    /// <summary>
    /// Обычно, пользовательских знакомест 16, и хотя адресоваться можно к любому, фактически их только 8. 
    /// Если DubAdrr - не отрицательные координаты, то они указывают на дублирующее знакоместа. 
    /// </summary>
    public Point DubAddr { get; set; }
    /// <summary>
    /// Двумерный массив точек символа
    /// Для клонирования пользоваться методами SetDots и GetDots
    /// </summary>
    public bool[,] Dots { get; set; }
    /// <summary>
    /// Сопоставление юникод-символу шрифта
    /// </summary>
    public char Symbol { get; set; }
    public LcdChar()
    {
        User = false;
        DubAddr = new Point(-1);
        Symbol = '\0';
        Dots = new bool[5, 8];
        for (int x = 0; x < 5; x++)
            for (int y = 0; y < 8; y++) Dots[x, y] = false;
    }

    public void SetDots(bool[,] dots)
    {
        Dots = (bool[,])dots.Clone();
    }
    public bool[,] GetDots()
    {
        return (bool[,])Dots.Clone();
    }
}
/// <summary>
/// Класс, описывающий шрифт ЖКИ (набор знакомест)
/// </summary>
[Serializable]
public class LcdFont
{
    /// <summary>
    /// Информация о версии для обеспечения дальнейшей совместимости
    /// </summary>
    public string Version { get; set; }
    /// <summary>
    /// Двумерный массив знакомест (символов)
    /// </summary>
    public LcdChar[,] Chars { get; set; }
    /// <summary>
    /// Наименование шрифта
    /// </summary>
    public string FontName { get; set; }
    /// <summary>
    /// Комментарии
    /// </summary>
    public string Comment { get; set; }
    /// <summary>
    /// Datasheet.pdf
    /// </summary>
    public MemoryStream PdfDatasheet { get; set; }
    public LcdFont()
    {
        Version = "0.0.1";
        FontName = " ";
        Comment = " ";
        PdfDatasheet = new MemoryStream();
        Chars = new LcdChar[16, 16];
        for (int x = 0; x < 16; x++)
            for (int y = 0; y < 16; y++) Chars[x, y] = new LcdChar();
    }
    /// <summary>
    /// Сохранение шрифта ЖКИ в MemoryStream
    /// </summary>
    /// <param name="name">Имя потока</param>
    /// <returns></returns>
    public bool SaveToStream(MemoryStream name)
    {
        try
        {
            BinaryWriter bwriter = new BinaryWriter(name);
            bwriter.Write(Version);
            for (int x = 0; x < 16; x++)
                for (int y = 0; y < 16; y++)
                {
                    bwriter.Write(Chars[x, y].User);
                    bwriter.Write(Chars[x, y].DubAddr.X);
                    bwriter.Write(Chars[x, y].DubAddr.Y);
                    bwriter.Write(Chars[x, y].Symbol);
                    for (int xx = 0; xx < 5; xx++)
                        for (int yy = 0; yy < 8; yy++) bwriter.Write(Chars[x, y].Dots[xx, yy]);
                }
            if (FontName.Length < 1) FontName = " ";
            bwriter.Write(FontName);
            if (Comment.Length < 1) Comment = " ";
            bwriter.Write(Comment);
            if (PdfDatasheet.Length > 0)
            {
                byte[] b = PdfDatasheet.ToArray();
                bwriter.Write(b.Length);
                bwriter.Write(b, 0, b.Length);
            }
            else
                bwriter.Write((Int32)0);
            bwriter.Close();
            return true;
        }
        catch {}
        return false;
    }
    /// <summary>
    /// Сохранение шрифта ЖКИ в файл
    /// </summary>
    /// <param name="fileName">Имя файла</param>
    /// <returns></returns>
    public bool SaveToFile(string fileName)
    {
        if (fileName == string.Empty) return false;
        MemoryStream mstream = new MemoryStream();
        if (SaveToStream(mstream))
        {
            try
            {
                FileStream file = new FileStream(fileName, FileMode.Create, FileAccess.Write);
                byte[] b = mstream.ToArray();
                file.Write(b, 0, b.Length);
                file.Close();
                mstream.Close();
                return true;
            }
            catch { }
        };
        return false;
    }
    /// <summary>
    /// Чтение шрифта ЖКИ из памяти
    /// </summary>
    /// <param name="name">Имя потока</param>
    /// <returns></returns>
    public bool ReadFromStream(MemoryStream name)
    {
        try
        {
            BinaryReader breader = new BinaryReader(name);
            breader.BaseStream.Position = 0;
            Version = breader.ReadString();
            for (int x = 0; x < 16; x++)
                for (int y = 0; y < 16; y++)
                {
                    Chars[x, y].User = breader.ReadBoolean();
                    int X = breader.ReadInt32();
                    int Y = breader.ReadInt32();
                    Chars[x, y].DubAddr = new Point(X, Y);
                    Chars[x, y].Symbol = breader.ReadChar();
                    for (int xx = 0; xx < 5; xx++)
                        for (int yy = 0; yy < 8; yy++) Chars[x, y].Dots[xx, yy] = breader.ReadBoolean();
                }
            FontName = breader.ReadString();
            Comment = breader.ReadString();            
            int len = breader.ReadInt32();
            if (len > 0)
            {
                byte[] b = breader.ReadBytes(len);
                PdfDatasheet.Write(b, 0, len);
            }
            breader.Close();
            return true;
        }
        catch //(Exception e) 
        {
            return false;
        }
    }
    /// <summary>
    /// Чтение шрифта ЖКИ из файла
    /// </summary>
    /// <param name="fileName">Имя файла</param>
    /// <returns></returns>
    public bool ReadFromFile(string fileName)
    {
        if (fileName == string.Empty) return false;
        try
        {
            FileStream fstream = new FileStream(fileName, FileMode.Open, FileAccess.Read);
            MemoryStream mstream = new MemoryStream();
            byte[] b = new byte[fstream.Length];
            fstream.Read(b, 0, (int)fstream.Length);
            mstream.Write(b, 0, (int)fstream.Length); 
            if (ReadFromStream(mstream))
            {
                fstream.Close();
                mstream.Close();
                return true;
            }
        }
        catch {}
        return false;
    }
}
 